import { buntstift } from 'buntstift';
import { dependencyCheckTask } from '../../tasks/dependencyCheckTask';
import { exit } from '../../utils/exit';
import { getPackageRoot } from '../../utils/getPackageRoot';
import { runPreOrPostScript } from '../../tasks/runPreOrPostScript';
const depsCommand = function () {
    return {
        name: 'deps',
        description: 'Check for missing, outdated, and unused dependencies.',
        optionDefinitions: [],
        async handle({ options: { verbose } }) {
            buntstift.configure(buntstift.getConfiguration().
                withVerboseMode(verbose));
            const applicationRootResult = await getPackageRoot({
                directory: process.cwd()
            });
            if (applicationRootResult.hasError()) {
                buntstift.error('Roboter must be run in an npm project.');
                return exit(1);
            }
            const applicationRoot = applicationRootResult.value;
            await runPreOrPostScript({
                applicationRoot,
                task: 'deps',
                phase: 'pre'
            });
            const dependencyCheckResult = await dependencyCheckTask({ applicationRoot });
            if (dependencyCheckResult.hasError()) {
                return exit(1);
            }
            await runPreOrPostScript({
                applicationRoot,
                task: 'deps',
                phase: 'post'
            });
        }
    };
};
export { depsCommand };
